<?php

namespace Drp\AdfsAuthentication\Exceptions;

class AdfsServerException extends \Exception
{
    /**
     * @var int
     */
    private $httpStatusCode;

    /**
     * @var string
     */
    private $errorType;

    /**
     * @var null|string
     */
    private $hint;

    /**
     * Throw a new exception.
     *
     * @param string $message Error message
     * @param int $code Error code
     * @param string $errorType Error type
     * @param int $httpStatusCode HTTP status code to send (default = 400)
     * @param null|string $hint A helper hint
     */
    public function __construct($message, $code, $errorType, $httpStatusCode = 400, $hint = null)
    {
        parent::__construct($message, $code);
        $this->httpStatusCode = $httpStatusCode;
        $this->errorType = $errorType;
        $this->hint = $hint;
    }

    /**
     * Invalid credentials error.
     *
     * @return static
     */
    public static function invalidCredentials()
    {
        return new static(
            'The user credentials were incorrect.',
            6,
            'invalid_credentials',
            401
        );
    }

    /**
     * Server error.
     *
     * @return static
     *
     * @codeCoverageIgnore
     */
    public static function serverError()
    {
        return new static(
            'The server encountered an unexpected condition which prevented it from fulfilling the request.',
            7,
            'server_error',
            500
        );
    }

    /**
     * Invalid Server Response error.
     *
     * @return static
     *
     * @codeCoverageIgnore
     */
    public static function invalidResponse()
    {
        return new static(
            'The authorization server returned a malformed status and cannot continue. ',
            7,
            'invalid_response',
            500
        );
    }

    /**
     * Unknown error.
     *
     * @return static
     *
     * @codeCoverageIgnore
     */
    public static function unknownError()
    {
        return new static(
            'An Unknown Error Occurred.',
            7,
            'server_error',
            500
        );
    }

    /**
     * @return string
     */
    public function getErrorType(): string
    {
        return $this->errorType;
    }

    /**
     * Get all headers that have to be send with the error response.
     *
     * @return array Array with header values
     */
    public function getHttpHeaders()
    {
        return [
            'Content-type' => 'application/json',
        ];
    }

    /**
     * Returns the HTTP status code to send when the exceptions is output.
     *
     * @return int
     */
    public function getHttpStatusCode()
    {
        return $this->httpStatusCode;
    }

    /**
     * @return null|string
     */
    public function getHint()
    {
        return $this->hint;
    }
}
